import React, { useContext, useState } from 'react';
import { AppContext } from '../../layout/AppContext';
import { haveSameDay } from '../../utils/date';
import { CaloriesInput } from '../caloriesInput/CaloriesInput';
import { CalorieRow } from './CalorieRow';
import styles from './caloriesList.css';
import { ItemActions } from './ItemActions';
import { ListEditBox } from './ListEditBox';

function CaloriesList () {
  const {calorieItems = [], setCalorieItems, selectedDate} = useContext(AppContext);
  const [selectedItem, setSelectedItem] = useState<CalorieValue>();

  function addCalories (addedCalories: { title: string, count: number }) {
    const updatedCalories = [
      ...calorieItems,
      {
        ...addedCalories,
        timestamp: selectedDate
      }
    ];

    setCalorieItems(updatedCalories);
  }

  function removeItem () {
    setCalorieItems(calorieItems.filter((item) => item !== selectedItem));
  }

  function saveCalories (changedCalories: { title: string, count: number }) {
    if (!selectedItem) {
      return;
    }

    const itemIndex = calorieItems.indexOf(selectedItem);
    const updatedItem = {
      ...selectedItem,
      ...changedCalories
    };

    const updatedItems =  [
      ...calorieItems.slice(0, itemIndex),
      updatedItem,
      ...calorieItems.slice(itemIndex + 1)
    ];

    setCalorieItems(updatedItems);
    setSelectedItem(updatedItem);
  }

  function onRowClick (item: CalorieValue) {
    if (selectedItem === item) {
      return setSelectedItem(undefined);
    }

    return setSelectedItem(item);
  }

  const calories = calorieItems.filter(({timestamp}) => haveSameDay(timestamp, selectedDate));

  return (
    <section className={styles.caloriesList}>
      <section className={styles.caloriesItems}>
        {calories.map((item: CalorieValue, index: number) => {
          // todo: use better key
          const isSelected = item === selectedItem;
          return (
            <CalorieRow key={index} {...{item, onRowClick, isSelected}} />
          );
        })}
      </section>

      <div className={styles.editBoxes}>
        {selectedItem &&
        <ListEditBox>
          <ItemActions {...{removeItem}} />
        </ListEditBox>
        }

        <ListEditBox>
          <CaloriesInput {...{addCalories, saveCalories, item: selectedItem}} />
        </ListEditBox>
      </div>
    </section>
  );
}

export {
  CaloriesList
};
